;**********************************************************************************
;Title:			stepper motor controller code
;Version:			1.0
;Last update:		2002.02.23
;Target MCU:		AT90S2313
; support E-mail:		mindtan@yahoo.com
;support website:		www.geocities.com/mindtan2000.html
;Low Register Usage:	1
;High Register Usage:	7
;Interrupt Usage:		External Interrupt (INT0)
;total memory space
;after compile to hex	:104 byte (1 byte = 8 bit)
;
;
;Description
;
;This source code shown how to make a simple stepper motor controller using external
;interrupt method (pin INT0). The port B (bit 4 to 7) used as step sequence
;pins. The step sequence is simple full step with one wave algorithm only.
;**********************************************************************************

.include "2313def.inc"

.def	u_sr=r15
.def	set_output=r16
.def	clearValue=r17
.def	tmp=r18
.def	firt_value=r19
.def	last_value=r20
.def	result=r21
.def	u_bit=r22
.equ	DIR=3
.equ	SCK=2
;----------------------------------------------------------------------------------
;Program start here what ever power firt time on or watchdog timer reset
;----------------------------------------------------------------------------------
.cseg
.org 	$000			;reset vector
	rjmp	start
.org	INT0addr		;INT0 interrupt vector ($001)
	rjmp	INT_SERV


;--------------------------------------------------------------------------------
;subrotine: hardwave setting, initial parameter setting
;--------------------------------------------------------------------------------
Port_Init:
	ldi 	r16, low(RAMEND)
	out	SPL,r16
	ldi	tmp,$00		;set port D as input
	out	DDRD,tmp		
	ldi	result,$ff		;set port B as output
	out	DDRB,result
	ldi	r23,$40		;enable INT0 interrupt setting
	out	GIMSK,r23
	ldi	r23,$03		;set rising edge of INT0 intterupt request
	out	MCUCR,r23
	ldi	last_value,$10
	ldi	firt_value,$80
	ldi	clearValue,$00
	ldi	set_output,$10
	out	PORTB,set_output
	ret
	
;---------------------------------------------------------------------------------
;subrotine:External intterrupt service routine (INT0)
;---------------------------------------------------------------------------------
INT_SERV:
	in	u_sr, SREG		;store status register (SREG)
	sbis	PIND,DIR		;check stepper direction pin
	rjmp	forward		;direction pin not set, jamp to forward
	rjmp	reverse		;direction pin set, jamp to reverse 
forward:
	rcall	run_forward	
	rjmp	exit
reverse:
	rcall	run_reverse
exit:
	out	SREG, u_sr		;restore SREG
	reti

;*********************************************************************************
;subrotine : stepper motor change step here (full step, one wave)
;*********************************************************************************
run_forward:
	cp	set_output, clearValue	;compare between two content(data)
	breq	reset_output_f		;jamp to reset_output_f if equal
	cp	set_output, firt_value	;compare between two content(data)
	brne	rlf_output			;jamp to rlf_output if not equal
	rjmp	reset_output_f
rlf_output:	
	lsl	set_output
	out	PORTB,set_output
	ret
reset_output_f:
	ldi	set_output,$10
	out	PORTB,set_output
	ret

run_reverse:
	cp	set_output, clearValue	;compare between two content(data)
	breq	reset_output_r		;jamp to reset_output_r if equal
	cp	set_output, last_value	;compare between two content(data)
	brne	rrf_output			;jamp to rrf_output if equal
	rjmp	reset_output_r
rrf_output:	
	lsr	set_output
	out	PORTB,set_output
	ret
reset_output_r:
	ldi	set_output,$80
	out	PORTB,set_output
	ret


;----------------------------------------------------------------------------------
;main program routine
;----------------------------------------------------------------------------------	
start:
	rcall	Port_Init	;call hardware setting subrotine
reloop:
	sei			;enable global interrupt
	rjmp	reloop
	
	